from i_editor import IEditor
from editor import Editor

from i_editor_manager import IEditorManager
from editor_manager import EditorManager

from i_perspective import IPerspective
from perspective import Perspective
from perspective_item import PerspectiveItem

from i_view import IView
from view import View

from i_workbench import IWorkbench
from workbench import Workbench

from workbench_window import WorkbenchWindow

from traits_ui_editor import TraitsUIEditor
from traits_ui_view import TraitsUIView
