#-----------------------------------------------------------------------------
#
#  Copyright (c) 2006 by Enthought, Inc.
#  All rights reserved.
#
#  Author: Dave Peterson <dpeterson@enthought.com>
#
#-----------------------------------------------------------------------------

# IDs of services provided by this plugin
from services import IPROJECT_MODEL, IPROJECT_UI

# Commonly referred to classes within this plugin
from factory_definition import FactoryDefinition
from model_service import ModelService
from project import Project
from project_action import ProjectAction
from project_factory import ProjectFactory
from view.project_view import ProjectView
# FIXME: Add back this import when it actually works :)
#from editor.project_editor import ProjectEditor

#### EOF #####################################################################
