from new_project_action import NewProjectAction
from open_project_action import OpenProjectAction
from save_project_action import SaveProjectAction
from save_as_project_action import SaveAsProjectAction
from close_project_action import CloseProjectAction
from switch_to_action import SwitchToAction
